@@ -21,19 +21,18 @@ __all__ = ["ML_Error_Regimes"]
 
 
 class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
-
     """
     ML estimation of the spatial error model with regimes (note no consistency 
     checks, diagnostics or constants added); :cite:`Anselin1988`
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    regimes      : list
+    regimes      : list or pandas.Series
                    List of n values with the mapping of each
                    observation to a regime. Assumed to be aligned with 'x'.
     constant_regi: string
@@ -54,7 +53,7 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
                    Spatial weights sparse matrix 
     slx_lags     : integer
                    Number of spatial lags of X to include in the model specification.
-                   If slx_lags>0, the specification becomes of the SDEM type.                          
+                   If slx_lags>0, the specification becomes of the SLX-Error type.                          
     method       : string
                    if 'full', brute force calculation (full matrix expressions)
                    if 'ord', Ord eigenvalue computation
@@ -83,7 +82,7 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
     name_regimes : string
                    Name of regimes variable for use in output
     latex        : boolean
-                   Specifies if summary is to be printed in latex format
+                   Specifies if the table with the coefficients' results and their inference is to be printed in LaTeX format
 
     Attributes
     ----------
@@ -300,14 +299,14 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
         name_regimes=None,
         latex=False,
     ):
-
         n = USER.check_arrays(y, x)
-        y = USER.check_y(y, n)
-        USER.check_weights(w, y, w_required=True)
+        y, name_y = USER.check_y(y, n, name_y)
+        w = USER.check_weights(w, y, w_required=True, slx_lags=slx_lags)
         self.constant_regi = constant_regi
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_y = USER.set_name_y(name_y)
         self.name_w = USER.set_name_w(name_w, w)
+        regimes, name_regimes = USER.check_reg_list(regimes, name_regimes, n)
         self.name_regimes = USER.set_name_ds(name_regimes)
         self.n = n
         self.y = y
@@ -316,7 +315,7 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
         set_warn(self, warn)
         name_x = USER.set_name_x(name_x, x_constant, constant=True)
 
-        if slx_lags >0:
+        if slx_lags > 0:
             lag_x = get_lags(w, x_constant, slx_lags)
             x_constant = np.hstack((x_constant, lag_x))
             name_x += USER.set_name_spatial_lags(name_x, slx_lags)
@@ -324,7 +323,7 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
         self.name_x_r = USER.set_name_x(name_x, x_constant)
 
         cols2regi = REGI.check_cols2regi(constant_regi, cols2regi, x_constant)
-        self.cols2regi = cols2regi        
+        self.cols2regi = cols2regi
         self.regimes_set = REGI._get_regimes_set(regimes)
         self.regimes = regimes
         USER.check_regimes(self.regimes_set, self.n, x.shape[1])
@@ -364,7 +363,7 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
                 constant_regi=None,
                 cols2regi=cols2regi,
                 names=name_x,
-                rlist=True
+                rlist=True,
             )
             BaseML_Error.__init__(
                 self,
@@ -377,8 +376,8 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
             )
 
             self.title = "ML SPATIAL ERROR"
-            if slx_lags >0:
-                self.title += " WITH SLX (SDEM)"
+            if slx_lags > 0:
+                self.title += " WITH SLX (SLX-Error)"
             self.title += " - REGIMES (METHOD = " + method + ")"
 
             self.name_x = USER.set_name_x(name_x, x, constant=True)
@@ -387,17 +386,28 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
             self.chow = REGI.Chow(self)
             self.aic = DIAG.akaike(reg=self)
             self.schwarz = DIAG.schwarz(reg=self)
-            self.output = pd.DataFrame(self.name_x, columns=['var_names'])
-            self.output['var_type'] = ['x'] * (len(self.name_x) - 1) + ['lambda']
-            self.output['regime'] = x_rlist + ['_Global']
-            self.output['equation'] = 0
+            self.output = pd.DataFrame(self.name_x, columns=["var_names"])
+            self.output["var_type"] = ["x"] * (len(self.name_x) - 1) + ["lambda"]
+            self.output["regime"] = x_rlist + ["_Global"]
+            self.output["equation"] = 0
             self.other_top = _nonspat_top(self, ml=True)
             output(reg=self, vm=vm, robust=False, other_end=False, latex=latex)
 
     def _error_regimes_multi(
-        self, y, x, regimes, w, slx_lags, cores, method, epsilon, cols2regi, vm, name_x, latex
+        self,
+        y,
+        x,
+        regimes,
+        w,
+        slx_lags,
+        cores,
+        method,
+        epsilon,
+        cols2regi,
+        vm,
+        name_x,
+        latex,
     ):
-
         regi_ids = dict(
             (r, list(np.where(np.array(regimes) == r)[0])) for r in self.regimes_set
         )
@@ -475,7 +485,9 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
 
         results = {}
         counter = 0
-        self.output = pd.DataFrame(columns=['var_names', 'var_type', 'regime', 'equation'])
+        self.output = pd.DataFrame(
+            columns=["var_names", "var_type", "regime", "equation"]
+        )
         for r in self.regimes_set:
             """
             if is_win:
@@ -492,24 +504,30 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
                 (counter * self.kr) : ((counter + 1) * self.kr),
                 (counter * self.kr) : ((counter + 1) * self.kr),
             ] = results[r].vm
-            self.betas[
-                (counter * self.kr) : ((counter + 1) * self.kr),
-            ] = results[r].betas
-            self.u[
-                regi_ids[r],
-            ] = results[r].u
-            self.predy[
-                regi_ids[r],
-            ] = results[r].predy
-            self.e_filtered[
-                regi_ids[r],
-            ] = results[r].e_filtered
+            self.betas[(counter * self.kr) : ((counter + 1) * self.kr),] = results[
+                r
+            ].betas
+            self.u[regi_ids[r],] = results[r].u
+            self.predy[regi_ids[r],] = results[r].predy
+            self.e_filtered[regi_ids[r],] = results[r].e_filtered
             self.name_y += results[r].name_y
             self.name_x += results[r].name_x
             results[r].other_top = _nonspat_top(results[r], ml=True)
-            self.output = pd.concat([self.output, pd.DataFrame({'var_names': results[r].name_x,
-                                                                'var_type': ['x'] * (len(results[r].name_x) - 1) + ['lambda'],
-                                                                'regime': r, 'equation': r})], ignore_index=True)
+            self.output = pd.concat(
+                [
+                    self.output,
+                    pd.DataFrame(
+                        {
+                            "var_names": results[r].name_x,
+                            "var_type": ["x"] * (len(results[r].name_x) - 1)
+                            + ["lambda"],
+                            "regime": r,
+                            "equation": r,
+                        }
+                    ),
+                ],
+                ignore_index=True,
+            )
             counter += 1
         self.chow = REGI.Chow(self)
         self.multi = results
@@ -517,7 +535,19 @@ class ML_Error_Regimes(BaseML_Error, REGI.Regimes_Frame):
 
 
 def _work_error(
-    y, x, regi_ids, r, w, slx_lags, method, epsilon, name_ds, name_y, name_x, name_w, name_regimes
+    y,
+    x,
+    regi_ids,
+    r,
+    w,
+    slx_lags,
+    method,
+    epsilon,
+    name_ds,
+    name_y,
+    name_x,
+    name_w,
+    name_regimes,
 ):
     w_r, warn = REGI.w_regime(w, regi_ids[r], r, transform=True)
     y_r = y[regi_ids[r]]
@@ -526,8 +556,8 @@ def _work_error(
     set_warn(model, warn)
     model.w = w_r
     model.title = "ML SPATIAL ERROR"
-    if slx_lags >0:
-        model.title += " WITH SLX (SDEM)"
+    if slx_lags > 0:
+        model.title += " WITH SLX (SLX-Error)"
     model.title += " - REGIME " + str(r) + " (METHOD = " + method + ")"
     model.name_ds = name_ds
     model.name_y = "%s_%s" % (str(r), name_y)
