@@ -2,15 +2,15 @@
 Spatial Two Stages Least Squares
 """
 
-__author__ = "Luc Anselin luc.anselin@asu.edu, David C. Folch david.folch@asu.edu"
+__author__ = "Luc Anselin lanselin@gmail.com, David C. Folch david.folch@asu.edu"
 
 import numpy as np
 from . import twosls as TSLS
 from . import user_output as USER
 from .utils import set_endog, sp_att, set_warn
-from .sputils import _spmultiplier
 import pandas as pd
 from .output import output, _spat_diag_out, _spat_pseudo_r2, _summary_impacts
+from itertools import compress
 
 __all__ = ["GM_Lag"]
 
@@ -48,6 +48,8 @@ class BaseGM_Lag(TSLS.BaseTSLS):
     slx_lags     : integer
                    Number of spatial lags of X to include in the model specification.
                    If slx_lags>0, the specification becomes of the Spatial Durbin type.
+    slx_vars     : either "All" (default) or list of booleans to select x variables
+                   to be lagged
     robust       : string
                    If 'white', then a White consistent estimator of the
                    variance-covariance matrix is given.  If 'hac', then a
@@ -177,18 +179,23 @@ class BaseGM_Lag(TSLS.BaseTSLS):
             w=None,
             w_lags=1,
             slx_lags=0,
+            slx_vars="All",
             lag_q=True,
             robust=None,
             gwk=None,
             sig2n_k=False,
     ):
+        
+
 
         if slx_lags > 0:
-            yend2, q2, wx = set_endog(y, x[:, 1:], w, yend, q, w_lags, lag_q, slx_lags)
+            yend2, q2, wx = set_endog(y, x[:, 1:], w, yend, q, w_lags, lag_q, slx_lags,slx_vars)
             x = np.hstack((x, wx))
         else:
             yend2, q2 = set_endog(y, x[:, 1:], w, yend, q, w_lags, lag_q)
 
+        
+
         TSLS.BaseTSLS.__init__(
             self, y=y, x=x, yend=yend2, q=q2, robust=robust, gwk=gwk, sig2n_k=sig2n_k
         )
@@ -201,15 +208,15 @@ class GM_Lag(BaseGM_Lag):
 
     Parameters
     ----------
-    y            : array
+    y            : numpy.ndarray or pandas.Series
                    nx1 array for dependent variable
-    x            : array
+    x            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    independent (exogenous) variable, excluding the constant
-    yend         : array
+    yend         : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    endogenous variable
-    q            : array
+    q            : numpy.ndarray or pandas object
                    Two dimensional array with n rows and one column for each
                    external exogenous variable to use as instruments (note:
                    this should not contain any variables from x); cannot be
@@ -226,6 +233,8 @@ class GM_Lag(BaseGM_Lag):
     slx_lags     : integer
                    Number of spatial lags of X to include in the model specification.
                    If slx_lags>0, the specification becomes of the Spatial Durbin type.
+    slx_vars     : either "All" (default) or list of booleans to select x variables
+                   to be lagged
     robust       : string
                    If 'white', then a White consistent estimator of the
                    variance-covariance matrix is given.  If 'hac', then a
@@ -238,10 +247,10 @@ class GM_Lag(BaseGM_Lag):
                    If True, then use n-k to estimate sigma^2. If False, use n.
     spat_diag    : boolean
                    If True, then compute Anselin-Kelejian test and Common Factor Hypothesis test (if applicable)
-    spat_impacts : string
+    spat_impacts : string or list
                    Include average direct impact (ADI), average indirect impact (AII),
                     and average total impact (ATI) in summary results.
-                    Options are 'simple', 'full', 'power', or None.
+                    Options are 'simple', 'full', 'power', 'all' or None.
                     See sputils.spmultiplier for more information.
     vm           : boolean
                    If True, include variance-covariance matrix in summary
@@ -367,7 +376,8 @@ class GM_Lag(BaseGM_Lag):
                    :math:`Z'H(H'H)^{-1}`
     pfora1a2     : array
                    n(zthhthi)'varb
-
+    sp_multipliers: dict
+                   Dictionary of spatial multipliers (if spat_impacts is not None)
 
     Examples
     --------
@@ -503,6 +513,7 @@ class GM_Lag(BaseGM_Lag):
             w_lags=1,
             lag_q=True,
             slx_lags=0,
+            slx_vars="All",
             robust=None,
             gwk=None,
             sig2n_k=False,
@@ -521,22 +532,33 @@ class GM_Lag(BaseGM_Lag):
     ):
 
         n = USER.check_arrays(x, yend, q)
-        y = USER.check_y(y, n)
-        USER.check_weights(w, y, w_required=True)
+        y, name_y = USER.check_y(y, n, name_y)
+        w = USER.check_weights(w, y, w_required=True, slx_lags=slx_lags)
         USER.check_robust(robust, gwk)
-        if robust == "hac" and spat_diag:
-            set_warn(
-                self,
-                "Spatial diagnostics are not available for HAC estimation. The spatial diagnostics have been disabled for this model.",
-            )
-            spat_diag = False
+        yend, q, name_yend, name_q = USER.check_endog([yend, q], [name_yend, name_q])
+        spat_diag, warn = USER.check_spat_diag(spat_diag=spat_diag, w=w, robust=robust, slx_lags=slx_lags)
+        set_warn(self, warn)
         x_constant, name_x, warn = USER.check_constant(x, name_x)
+        set_warn(self, warn)
         name_x = USER.set_name_x(name_x, x_constant)  # need to check for None and set defaults
 
-        if slx_lags > 0:
-            name_x += USER.set_name_spatial_lags(name_x[1:], slx_lags)  # exclude constant
+        # kx and wkx are used to replace complex calculation for output
+        if slx_lags > 0:  # adjust for flexwx
+            if (isinstance(slx_vars,list)):     # slx_vars has True,False
+                if len(slx_vars) != x.shape[1] :
+                    raise Exception("slx_vars incompatible with x column dimensions")
+                else:  # use slx_vars to extract proper columns
+                    workname = name_x[1:]
+                    kx = len(workname)
+                    vv = list(compress(workname,slx_vars))
+                    name_x += USER.set_name_spatial_lags(vv, slx_lags)
+                    wkx = slx_vars.count(True)
+            else:
+                kx = len(name_x) - 1
+                wkx = kx
+                name_x += USER.set_name_spatial_lags(name_x[1:], slx_lags)  # exclude constant
+                
 
-        set_warn(self, warn)
         BaseGM_Lag.__init__(
             self,
             y=y,
@@ -546,11 +568,13 @@ class GM_Lag(BaseGM_Lag):
             q=q,
             w_lags=w_lags,
             slx_lags=slx_lags,
+            slx_vars=slx_vars,
             robust=robust,
             gwk=gwk,
             lag_q=lag_q,
             sig2n_k=sig2n_k,
         )
+
         self.rho = self.betas[-1]
         self.predy_e, self.e_pred, warn = sp_att(
             w, self.y, self.predy, self.yend[:, -1].reshape(self.n, 1), self.rho, hard_bound=hard_bound
@@ -567,21 +591,37 @@ class GM_Lag(BaseGM_Lag):
         self.name_yend.append(USER.set_name_yend_sp(self.name_y))
         self.name_z = self.name_x + self.name_yend
         self.name_q = USER.set_name_q(name_q, q)
+
         if slx_lags > 0:  # need to remove all but last SLX variables from name_x
             self.name_x0 = []
             self.name_x0.append(self.name_x[0])  # constant
-            kx = int((self.k - self.kstar - 1) / (slx_lags + 1))  # number of original exogenous vars
-            self.name_x0.extend(self.name_x[-kx:])
+            if (isinstance(slx_vars,list)):   # boolean list passed
+                # x variables that were not lagged
+                self.name_x0.extend(list(compress(self.name_x[1:],[not i for i in slx_vars])))
+                # last wkx variables
+                self.name_x0.extend(self.name_x[-wkx:])
+
+
+            else:
+                okx = int((self.k - self.kstar - 1) / (slx_lags + 1))  # number of original exogenous vars
+
+                self.name_x0.extend(self.name_x[-okx:])
+
             self.name_q.extend(USER.set_name_q_sp(self.name_x0, w_lags, self.name_q, lag_q))
-            var_types = ['x'] * (kx + 1) + ['wx'] * kx * slx_lags + ['yend'] * (len(self.name_yend) - 1) + ['rho']
+
+            #var_types = ['x'] * (kx + 1) + ['wx'] * kx * slx_lags + ['yend'] * (len(self.name_yend) - 1) + ['rho']
+            var_types = ['x'] * (kx + 1) + ['wx'] * wkx * slx_lags + ['yend'] * (len(self.name_yend) - 1) + ['rho']
         else:
             self.name_q.extend(USER.set_name_q_sp(self.name_x, w_lags, self.name_q, lag_q))
             var_types = ['x'] * len(self.name_x) + ['yend'] * (len(self.name_yend) - 1) + ['rho']
+
         self.name_h = USER.set_name_h(self.name_x, self.name_q)
         self.robust = USER.set_robust(robust)
         self.name_w = USER.set_name_w(name_w, w)
         self.name_gwk = USER.set_name_w(name_gwk, gwk)
         self.slx_lags = slx_lags
+        self.slx_vars = slx_vars
+
         self.output = pd.DataFrame(self.name_x + self.name_yend, columns=['var_names'])
         self.output['var_type'] = var_types
         self.output['regime'], self.output['equation'] = (0, 0)
@@ -590,12 +630,12 @@ class GM_Lag(BaseGM_Lag):
 
         if spat_diag:
             diag_out = _spat_diag_out(self, w, 'yend')
-        if spat_impacts and slx_lags == 0:
-            impacts = _summary_impacts(self, _spmultiplier(w, self.rho, method=spat_impacts), spat_impacts, slx_lags)
+        if spat_impacts:
+            self.sp_multipliers, impacts_str = _summary_impacts(self, w, spat_impacts, slx_lags,slx_vars)
             try:
-                diag_out += impacts
+                diag_out += impacts_str
             except TypeError:
-                diag_out = impacts
+                diag_out = impacts_str
         output(reg=self, vm=vm, robust=robust, other_end=diag_out, latex=latex)
 
 
